@@ -1,20 +1,22 @@
-// Copyright 2020-2021 Kevin Reid under the terms of the MIT License as detailed
+// Copyright 2020-2022 Kevin Reid under the terms of the MIT License as detailed
 // in the accompanying file README.md or <https://opensource.org/licenses/MIT>.
 
 //! A voxel reinterpretation of the famous Sponza Atrium test scene.
 
-use all_is_cubes::inv::Tool;
-use all_is_cubes::rgba_const;
 use noise::Seedable;
 
-use all_is_cubes::block::{space_to_blocks, Block, BlockAttributes, BlockCollision, AIR};
+use all_is_cubes::block::{
+    space_to_blocks, Block, BlockAttributes, BlockCollision, RotationPlacementRule, AIR,
+};
 use all_is_cubes::cgmath::{EuclideanSpace as _, Point3, Transform, Vector3};
-use all_is_cubes::content::palette;
+use all_is_cubes::character::Spawn;
+use all_is_cubes::content::{free_editing_starter_inventory, palette};
 use all_is_cubes::linking::{BlockModule, BlockProvider, InGenError};
 use all_is_cubes::math::{
     Face, FaceMap, FreeCoordinate, GridCoordinate, GridMatrix, GridPoint, GridRotation, GridVector,
     NoiseFnExt as _, Rgb,
 };
+use all_is_cubes::rgba_const;
 use all_is_cubes::space::{Grid, GridArray, SetCubeError, Space, SpacePhysics};
 use all_is_cubes::universe::Universe;
 
@@ -61,17 +63,19 @@ pub(crate) fn atrium(universe: &mut Universe) -> Result<Space, InGenError> {
     };
 
     let mut space = Space::builder(space_grid)
-        .spawn_position(Point3::new(
-            0.5,
-            1.91 + FreeCoordinate::from(ceiling_height + 2),
-            10.0,
-        ))
+        .spawn({
+            // TODO: default_for_new_space isn't really doing anything for us here
+            let mut spawn = Spawn::default_for_new_space(space_grid);
+            spawn.set_eye_position(Point3::new(
+                0.5,
+                1.91 + FreeCoordinate::from(ceiling_height + 2),
+                10.0,
+            ));
+            spawn.set_inventory(free_editing_starter_inventory(true));
+            spawn
+        })
         .sky_color(Rgb::new(1.0, 1.0, 0.9843) * 4.0)
         .build_empty();
-    // TODO: builder method ... ?
-    space
-        .spawn_mut()
-        .set_inventory(vec![Tool::RemoveBlock { keep: true }.into()]);
 
     // Outer walls
     four_walls(
@@ -455,10 +459,15 @@ fn install_atrium_blocks(
             .build_empty();
             space.fill(space.grid(), |p| {
                 // Flip middle of first block, so that the arch to our left appears on it
-                let z_for_arch /* but not for bricks */ = if p.z < resolution_g / 2 { resolution_g - 1 - p.z } else { p.z };
+                let z_for_arch /* but not for bricks */ = if p.z < resolution_g / 2 {
+                    resolution_g - 1 - p.z
+                } else {
+                    p.z
+                };
                 let arch_z_doubled = z_for_arch * 2 - arch_center_z_doubled;
                 let arch_y_doubled = p.y * 2;
-                let distance_from_edge = p.x.min(resolution_g - 1 - p.x) as f32 / resolution_g as f32;
+                let distance_from_edge =
+                    p.x.min(resolution_g - 1 - p.x) as f32 / resolution_g as f32;
                 let r = (arch_z_doubled as f32 / arch_opening_width as f32)
                     .hypot(arch_y_doubled as f32 / (arch_opening_height as f32 * 2.));
                 if r < 1.0 {
@@ -466,13 +475,21 @@ fn install_atrium_blocks(
                     None
                 } else if r < 1.04 {
                     // Beveled edge
-                    if distance_from_edge < ((1. - r) * 2.0 + 0.1) { None } else { Some(&stone_range[3]) }
+                    if distance_from_edge < ((1. - r) * 2.0 + 0.1) {
+                        None
+                    } else {
+                        Some(&stone_range[3])
+                    }
                 } else if r < 1.1 {
                     // Surface
                     Some(&stone_range[3])
                 } else if r < 1.14 {
                     // Groove
-                    if distance_from_edge == 0. { None } else { Some(&stone_range[4]) }
+                    if distance_from_edge == 0. {
+                        None
+                    } else {
+                        Some(&stone_range[4])
+                    }
                 } else {
                     // Body
                     Some(brick_pattern(p))
@@ -528,6 +545,7 @@ fn install_atrium_blocks(
             AtriumBlocks::GroundColumn => Block::builder()
                 .display_name("Large Atrium Column")
                 .collision(BlockCollision::Recur)
+                .rotation_rule(RotationPlacementRule::Attach { by: Face::NY })
                 .voxels_fn(universe, resolution, |p| {
                     let mid = (p * 2 - center_point_doubled).map(|c| c.abs());
                     if mid.x + mid.z < resolution_g * 6 / 4 {
@@ -540,6 +558,7 @@ fn install_atrium_blocks(
             AtriumBlocks::SquareColumn => Block::builder()
                 .display_name("Square Atrium Column")
                 .collision(BlockCollision::Recur)
+                .rotation_rule(RotationPlacementRule::Attach { by: Face::NY })
                 .voxels_fn(universe, resolution, |p| {
                     let mid = (p * 2 - center_point_doubled).map(|c| c.abs());
                     if mid.x.max(p.z) < resolution_g * 6 / 4 {
@@ -552,6 +571,7 @@ fn install_atrium_blocks(
             AtriumBlocks::SmallColumn => Block::builder()
                 .display_name("Round Atrium Column")
                 .collision(BlockCollision::Recur)
+                .rotation_rule(RotationPlacementRule::Attach { by: Face::NY })
                 .voxels_fn(universe, resolution, |p| {
                     let mid = (p * 2 - center_point_doubled).map(|c| c.abs());
                     if mid.x.pow(2) + mid.z.pow(2) < (resolution_g * 3 / 4).pow(2) {
@@ -564,6 +584,7 @@ fn install_atrium_blocks(
             AtriumBlocks::Molding => Block::builder()
                 .display_name("Atrium Top Edge Molding")
                 .collision(BlockCollision::Recur)
+                // TODO: rotation rule
                 .voxels_fn(universe, resolution, molding_fn)?
                 .build(),
             AtriumBlocks::MoldingCorner => Block::builder()
