@@ -1,25 +1,25 @@
-// Copyright 2020-2021 Kevin Reid under the terms of the MIT License as detailed
+// Copyright 2020-2022 Kevin Reid under the terms of the MIT License as detailed
 // in the accompanying file README.md or <https://opensource.org/licenses/MIT>.
 
 //! A space with miscellaneous demonstrations/tests of functionality.
 //! The individual buildings/exhibits are defined in [`DEMO_CITY_EXHIBITS`].
 
+use futures_core::future::LocalBoxFuture;
 use instant::Instant;
 use noise::Seedable as _;
 
-use all_is_cubes::cgmath::{EuclideanSpace as _, One as _, Transform as _, Vector3};
+use all_is_cubes::cgmath::{EuclideanSpace as _, One as _, Vector3};
 use all_is_cubes::drawing::embedded_graphics::{
     geometry::Point,
     mono_font::{iso_8859_1::FONT_9X18_BOLD, MonoTextStyle},
-    prelude::{Dimensions, Transform},
     text::{Baseline, Text},
 };
 
-use all_is_cubes::block::{Block, Resolution};
-use all_is_cubes::block::{BlockAttributes, BlockCollision, AIR};
+use all_is_cubes::block::Block;
+use all_is_cubes::block::AIR;
 use all_is_cubes::character::Spawn;
 use all_is_cubes::content::palette;
-use all_is_cubes::drawing::{draw_to_blocks, VoxelBrush};
+use all_is_cubes::drawing::VoxelBrush;
 use all_is_cubes::inv::Slot;
 use all_is_cubes::inv::Tool;
 use all_is_cubes::linking::{BlockProvider, InGenError};
@@ -28,14 +28,20 @@ use all_is_cubes::math::{
     NoiseFnExt as _, Rgb,
 };
 use all_is_cubes::raycast::Raycaster;
-use all_is_cubes::space::{Grid, LightPhysics, SetCubeError, Space, SpacePhysics};
+use all_is_cubes::space::{Grid, LightPhysics, Space, SpacePhysics};
 use all_is_cubes::universe::Universe;
+use all_is_cubes::util::YieldProgress;
 
+use crate::clouds::clouds;
 use crate::{
-    logo_text, logo_text_extent, wavy_landscape, DemoBlocks, LandscapeBlocks, DEMO_CITY_EXHIBITS,
+    draw_text_in_blocks, logo_text, logo_text_extent, space_to_space_copy, wavy_landscape,
+    DemoBlocks, LandscapeBlocks, DEMO_CITY_EXHIBITS,
 };
 
-pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
+pub(crate) async fn demo_city(
+    universe: &mut Universe,
+    p: YieldProgress,
+) -> Result<Space, InGenError> {
     let start_city_time = Instant::now();
 
     let landscape_blocks = BlockProvider::<LandscapeBlocks>::using(universe)?;
@@ -52,7 +58,7 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
     let sky_height = 30;
     let ground_depth = 30; // TODO: wavy_landscape is forcing us to have extra symmetry here
     let underground_floor_y = -5;
-    let radius_xz = 60;
+    let radius_xz = 80;
     let grid = Grid::from_lower_upper(
         (-radius_xz, -ground_depth, -radius_xz),
         (radius_xz, sky_height, radius_xz),
@@ -62,9 +68,9 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
 
     // Prepare brushes.
     let lamp_brush = VoxelBrush::new(vec![
-        ((0, 0, 0), &demo_blocks[Lamppost]),
-        ((0, 1, 0), &demo_blocks[Lamppost]),
-        ((0, 2, 0), &demo_blocks[Lamppost]),
+        ((0, 0, 0), &demo_blocks[LamppostBase]),
+        ((0, 1, 0), &demo_blocks[LamppostSegment]),
+        ((0, 2, 0), &demo_blocks[LamppostTop]),
         ((0, 3, 0), &demo_blocks[Lamp]),
     ]);
 
@@ -76,20 +82,18 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
             // TODO: Add incremental spawn configuration to SpaceBuilder?
             let mut spawn = Spawn::default_for_new_space(grid);
             spawn.set_eye_position(grid.center() + Vector3::new(0.5, 2.91, 8.5));
-            spawn.set_flying(false);
             // Initial inventory contents. TODO: Make a better list.
-            let mut inventory = vec![Tool::RemoveBlock { keep: true }.into()];
+            let mut inventory = vec![
+                Tool::RemoveBlock { keep: true }.into(),
+                Tool::Jetpack { active: false }.into(),
+            ];
             for block in [
-                &landscape_blocks[Grass],
-                &landscape_blocks[Dirt],
                 &landscape_blocks[Stone],
+                &demo_blocks[GlassBlock],
                 &demo_blocks[Lamp],
-                &demo_blocks[Sconce],
-                &demo_blocks[Road],
-                &demo_blocks[Signboard],
                 &demo_blocks[Arrow],
             ] {
-                inventory.push(Slot::stack(100, Tool::Block(block.clone())));
+                inventory.push(Slot::stack(40, Tool::Block(block.clone())));
             }
             spawn.set_inventory(inventory);
             spawn
@@ -98,7 +102,9 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
 
     // Fill basic layers, underground and top
     space.fill_uniform(planner.y_range(-ground_depth, 0), &landscape_blocks[Stone])?;
+    p.progress(0.1).await;
     space.fill_uniform(planner.y_range(0, 1), &landscape_blocks[Grass])?;
+    p.progress(0.2).await;
 
     // Stray grass
     let grass_noise_v = noise::OpenSimplex::new().set_seed(0x21b5cc6b);
@@ -121,6 +127,7 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
             }
         },
     )?;
+    p.progress(0.3).await;
 
     // Roads and lamps
     for &face in &[Face::PX, Face::NX, Face::PZ, Face::NZ] {
@@ -193,6 +200,7 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
             )?;
         }
     }
+    p.progress(0.4).await;
 
     let blank_city_time = Instant::now();
     log::trace!(
@@ -208,20 +216,35 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
         [-exhibit_front_radius, sky_height, -exhibit_front_radius],
     );
     space.fill_uniform(landscape_region, AIR)?;
+    p.progress(0.5).await;
     wavy_landscape(landscape_region, &mut space, &landscape_blocks, 1.0)?;
     planner.occupied_plots.push(landscape_region);
 
+    // Clouds (needs to be done after landscape to not be overwritten)
+    // TODO: Enable this once transparency rendering is better.
+    if false {
+        clouds(planner.y_range(sky_height - 2, sky_height), &mut space, 0.1)?;
+    }
+
+    // TODO: Integrate logging and YieldProgress
     let landscape_time = Instant::now();
     log::trace!(
         "Landscape took {:.3} s",
         landscape_time.duration_since(blank_city_time).as_secs_f32()
     );
+    let [exhibits_progress, final_progress] = p.finish_and_cut(0.6).await.split(0.8);
 
     // Exhibits
-    for exhibit in DEMO_CITY_EXHIBITS.iter() {
+    for (exhibit, exhibit_progress) in DEMO_CITY_EXHIBITS
+        .iter()
+        .zip(exhibits_progress.split_evenly(DEMO_CITY_EXHIBITS.len()))
+    {
         let start_exhibit_time = Instant::now();
         let exhibit_space = (exhibit.factory)(exhibit, universe)
+            .await
             .expect("exhibit generation failure. TODO: place an error marker and continue instead");
+        exhibit_progress.progress(0.7).await;
+
         let exhibit_footprint = exhibit_space.grid();
 
         let enclosure_footprint = exhibit_footprint.expand(FaceMap::repeat(1));
@@ -242,6 +265,7 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
             ],
         );
         space.fill_uniform(enclosure, &demo_blocks[ExhibitBackground])?;
+        exhibit_progress.progress(0.9).await;
 
         // TODO: Add "entrances" so it's clear what the "front" of the exhibit is supposed to be.
 
@@ -251,46 +275,18 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
             0,
             exhibit_footprint.upper_bounds().z + 1,
         ]);
-        let name_block_resolution = 32;
-        let font = &FONT_9X18_BOLD;
-        let name_bottom_y =
-            (name_block_resolution - font.character_size.height as GridCoordinate) / 2;
-        let name_text = Text::with_baseline(
-            exhibit.name,
-            Point::new(0, -name_bottom_y),
-            MonoTextStyle::new(font, palette::ALMOST_BLACK),
-            Baseline::Bottom,
-        );
-        // TODO: This is an awful lot of code to benerate "text is centered on a number of whole blocks"
-        let name_width = name_text.bounding_box().size.width as GridCoordinate;
-        let name_width_in_blocks: GridCoordinate =
-            (name_width + name_block_resolution - 1) / name_block_resolution; // rounding up
-        let name_blocks = draw_to_blocks(
+        let truncated_name_grid = draw_text_in_blocks(
             universe,
-            name_block_resolution as Resolution,
-            0,
-            0..1,
-            BlockAttributes {
-                display_name: format!("Exhibit name {:?}", exhibit.name).into(),
-                collision: BlockCollision::Recur,
-                ..BlockAttributes::default()
-            },
-            &name_text.translate(Point::new(
-                ((name_width_in_blocks * name_block_resolution) - name_width) / 2,
-                0,
-            )),
-        )
-        .expect("name drawing failure");
-        // Truncate name to not overrun the exhibit itself
-        let truncated_name_grid = name_blocks
-            .grid()
-            .intersection(Grid::new([0, 0, 0], [exhibit_footprint.size().x + 3, 1, 1]))
-            .unwrap();
-        space_to_space_copy(
-            &name_blocks,
-            truncated_name_grid,
             &mut space,
+            32,
+            exhibit_footprint.size().x + 3,
             plot_transform * name_transform,
+            &Text::with_baseline(
+                exhibit.name,
+                Point::new(0, 0),
+                MonoTextStyle::new(&FONT_9X18_BOLD, palette::ALMOST_BLACK),
+                Baseline::Bottom,
+            ),
         )?; // TODO: on failure, place an error marker and continue
         space.fill_uniform(
             truncated_name_grid
@@ -316,6 +312,8 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
             exhibit.name,
             exhibit_time.as_secs_f32()
         );
+
+        exhibit_progress.progress(1.0).await;
     }
 
     if false {
@@ -332,6 +330,7 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
         GridMatrix::from_translation([0, 12, -radius_xz]),
         &mut space,
     )?;
+    final_progress.progress(0.5).await;
 
     // Enable light computation
     space.set_physics({
@@ -339,35 +338,16 @@ pub(crate) fn demo_city(universe: &mut Universe) -> Result<Space, InGenError> {
         p.light = SpacePhysics::default().light;
         p
     });
+    final_progress.progress(1.0).await;
 
     Ok(space)
 }
 
-// TODO: move this since it is a generally useful utility
-fn space_to_space_copy(
-    src: &Space,
-    src_grid: Grid,
-    dst: &mut Space,
-    src_to_dst_transform: GridMatrix,
-) -> Result<(), SetCubeError> {
-    // TODO: don't panic
-    let dst_to_src_transform = src_to_dst_transform.inverse_transform().unwrap();
-    let (block_rotation, _) = src_to_dst_transform
-        .decompose()
-        .expect("could not decompose transform");
-    dst.fill(src_grid.transform(src_to_dst_transform).unwrap(), |p| {
-        Some(
-            src[dst_to_src_transform.transform_cube(p)]
-                .clone()
-                .rotate(block_rotation),
-        )
-    })
-}
-
 #[allow(clippy::type_complexity)]
 pub(crate) struct Exhibit {
     pub name: &'static str,
-    pub factory: fn(&Exhibit, &mut Universe) -> Result<Space, InGenError>,
+    pub factory:
+        for<'a> fn(&'a Exhibit, &'a mut Universe) -> LocalBoxFuture<'a, Result<Space, InGenError>>,
 }
 
 /// Tracks available land while the city is being generated.
