@@ -1,4 +1,4 @@
-// Copyright 2020-2021 Kevin Reid under the terms of the MIT License as detailed
+// Copyright 2020-2022 Kevin Reid under the terms of the MIT License as detailed
 // in the accompanying file README.md or <https://opensource.org/licenses/MIT>.
 
 //! Miscellanous demonstrations of capability and manual test-cases.
@@ -7,26 +7,28 @@
 use std::convert::TryFrom;
 use std::f64::consts::PI;
 
+use all_is_cubes::block::{
+    space_to_blocks, AnimationHint, Block, BlockAttributes, BlockCollision, Resolution,
+    RotationPlacementRule, AIR,
+};
 use all_is_cubes::cgmath::{
     Basis2, ElementWise, EuclideanSpace as _, InnerSpace as _, Rad, Rotation as _, Rotation2,
     Vector2, Vector3,
 };
-use all_is_cubes::drawing::embedded_graphics::{
-    geometry::Point,
-    mono_font::{
-        iso_8859_1::{FONT_6X10, FONT_8X13_BOLD},
-        MonoTextStyle,
+use all_is_cubes::drawing::{
+    draw_to_blocks,
+    embedded_graphics::{
+        geometry::Point,
+        mono_font::{
+            iso_8859_1::{FONT_6X10, FONT_8X13_BOLD},
+            MonoTextStyle,
+        },
+        pixelcolor::Rgb888,
+        prelude::Size,
+        primitives::{PrimitiveStyle, Rectangle, StyledDrawable},
+        text::{Baseline, Text},
     },
-    pixelcolor::Rgb888,
-    prelude::Size,
-    primitives::{PrimitiveStyle, Rectangle, StyledDrawable},
-    text::{Baseline, Text},
 };
-
-use all_is_cubes::block::{
-    space_to_blocks, AnimationHint, Block, BlockAttributes, BlockCollision, Resolution, AIR,
-};
-use all_is_cubes::drawing::draw_to_blocks;
 use all_is_cubes::linking::{BlockProvider, InGenError};
 use all_is_cubes::math::{
     Face, FaceMap, FreeCoordinate, GridCoordinate, GridMatrix, GridPoint, GridRotation, GridVector,
@@ -43,7 +45,8 @@ use crate::{
 
 pub(crate) static DEMO_CITY_EXHIBITS: &[Exhibit] = &[
     KNOT,
-    TRANSPARENCY,
+    TRANSPARENCY_LARGE,
+    TRANSPARENCY_SMALL,
     COLLISION,
     TEXT,
     RESOLUTIONS,
@@ -56,10 +59,19 @@ pub(crate) static DEMO_CITY_EXHIBITS: &[Exhibit] = &[
     COLORS,
 ];
 
-const TRANSPARENCY: Exhibit = Exhibit {
+macro_rules! exhibit {
+    (const $id:ident, name: $name_text:literal, ($this_var:pat, $uni_var:pat) $body:block) => {
+        const $id: Exhibit = Exhibit {
+            name: $name_text,
+            factory: |$this_var, $uni_var| Box::pin(async move { $body }),
+        };
+    };
+}
+
+exhibit! {
+    const TRANSPARENCY_LARGE,
     name: "Transparency",
-    factory: |_this, _universe| {
-        // TODO: Add some partial-block transparency once we're any good at implementing it
+    (_this, _universe) {
         let mut space = Space::empty(Grid::new([-3, 0, -3], [7, 5, 7]));
 
         let colors = [
@@ -70,8 +82,10 @@ const TRANSPARENCY: Exhibit = Exhibit {
         ];
         let alphas = [0.25, 0.5, 0.75, 0.95];
         for (rot, color) in GridRotation::CLOCKWISE.iterate().zip(&colors) {
-            let windowpane =
-                Grid::from_lower_upper([-1, 0, 3], [2, alphas.len() as GridCoordinate, 4]);
+            let windowpane = Grid::from_lower_upper(
+                [-1, 0, 3],
+                [2, alphas.len() as GridCoordinate, 4]
+            );
             space.fill(
                 windowpane
                     .transform(rot.to_positive_octant_matrix(1))
@@ -85,12 +99,101 @@ const TRANSPARENCY: Exhibit = Exhibit {
         }
 
         Ok(space)
-    },
-};
+    }
+}
 
-const KNOT: Exhibit = Exhibit {
+exhibit! {
+    const TRANSPARENCY_SMALL,
+    name: "Voxel Transparency WIP",
+    (_this, universe) {
+        let footprint = Grid::new([0, 0, 0], [7, 4, 7]);
+        let pool =  Grid::new([1, 0, 1], [5, 2, 5]);
+        let mut space = Space::empty(footprint);
+
+        let water_voxel = Block::builder()
+            .color(rgba_const!(0.02, 0.04, 0.9, 0.5))
+            .collision(BlockCollision::None)
+            .build();
+        let water_surface_voxel = Block::builder()
+            .color(rgba_const!(0.5, 0.5, 0.72, 0.8))
+            .collision(BlockCollision::None)
+            .build();
+
+        let water_surface_block = Block::builder()
+            .collision(BlockCollision::Recur)
+            .voxels_fn(universe, 8, |p| {
+                match p.y {
+                    0..=3 => &water_voxel,
+                    4 => &water_surface_voxel,
+                    _ => &AIR,
+                }
+            })?
+            .build();
+
+        let window_block = {
+            let window_pane_resolution = 16;
+            let depth = 3;
+            let window_frame_block = Block::from(palette::ALMOST_BLACK);
+            let window_glass_surface_block = Block::from(rgba_const!(0.5, 0.72, 0.5, 0.6));
+            let window_glass_inner_block = Block::from(rgba_const!(0.7, 0.72, 0.7, 0.05));
+            let upper = window_pane_resolution as GridCoordinate - 1;
+
+            Block::builder()
+                .collision(BlockCollision::Recur)
+                .rotation_rule(RotationPlacementRule::Attach { by: Face::NZ })
+                .voxels_fn(universe, window_pane_resolution, |p| {
+                    if p.z >= depth {
+                        return &AIR;
+                    }
+                    if p.x == 0 || p.y == 0 || p.x == upper || p.y == upper {
+                        return &window_frame_block;
+                    }
+                    if p.z == depth - 1 || p.z == 0 {
+                        if p.x == 1 || p.y == 1 || p.x == upper - 1 || p.y == upper - 1 {
+                            &window_frame_block
+                        } else {
+                            &window_glass_surface_block
+                        }
+                    } else {
+                        &window_glass_inner_block
+                    }
+                })?
+                .build()
+        };
+
+        four_walls(
+            pool.expand(FaceMap::symmetric([1, 0, 1])),
+            |_origin, direction, _length, wall_excluding_corners| {
+                space.fill_uniform(
+                    wall_excluding_corners,
+                    window_block.clone().rotate(
+                        GridRotation::from_to(Face::PX, direction, Face::PY).unwrap()
+                    )
+                )?;
+                Ok::<(), InGenError>(())
+            },
+        )?;
+
+        space.fill_uniform(
+            pool.abut(Face::NY, 0).unwrap().abut(Face::PY, 1).unwrap(),
+            &water_voxel,
+        )?;
+        space.fill_uniform(
+            pool.abut(Face::PY, -1).unwrap(),
+            &water_surface_block,
+        )?;
+
+        let [floater] = make_some_voxel_blocks(universe);
+        space.set([3, 1, 3], floater)?;
+
+        Ok(space)
+    }
+}
+
+exhibit! {
+    const KNOT,
     name: "Knot",
-    factory: |this, universe| {
+    (this, universe) {
         let footprint = Grid::new([-2, -2, -1], [5, 5, 3]);
         let resolution = 16;
         let toroidal_radius = 24.;
@@ -157,12 +260,13 @@ const KNOT: Exhibit = Exhibit {
             universe.insert_anonymous(drawing_space),
         )?;
         Ok(space)
-    },
-};
+    }
+}
 
-const TEXT: Exhibit = Exhibit {
+exhibit! {
+    const TEXT,
     name: "Text",
-    factory: |_, universe| {
+    (_, universe) {
         let space = draw_to_blocks(
             universe,
             16,
@@ -180,12 +284,13 @@ const TEXT: Exhibit = Exhibit {
             ),
         )?;
         Ok(space)
-    },
-};
+    }
+}
 
-const ANIMATION: Exhibit = Exhibit {
+exhibit! {
+    const ANIMATION,
     name: "Animation",
-    factory: |_this, universe| {
+    (_this, universe) {
         let footprint = Grid::new([0, 0, 0], [3, 1, 2]);
         let mut space = Space::empty(footprint);
 
@@ -243,12 +348,13 @@ const ANIMATION: Exhibit = Exhibit {
         space.set([2, 0, 0], fire_block)?;
 
         Ok(space)
-    },
-};
+    }
+}
 
-const COLLISION: Exhibit = Exhibit {
+exhibit! {
+    const COLLISION,
     name: "Collision WIP",
-    factory: |_this, universe| {
+    (_this, universe) {
         let half_block = make_slab(universe, 2, 4);
 
         let footprint = Grid::new([0, 0, 0], [5, 2, 4]);
@@ -282,12 +388,13 @@ const COLLISION: Exhibit = Exhibit {
         }
 
         Ok(space)
-    },
-};
+    }
+}
 
-const RESOLUTIONS: Exhibit = Exhibit {
+exhibit! {
+    const RESOLUTIONS,
     name: "Resolutions",
-    factory: |_this, universe| {
+    (_this, universe) {
         let footprint = Grid::new([0, 0, 0], [5, 2, 3]);
         let mut space = Space::empty(footprint);
 
@@ -338,12 +445,13 @@ const RESOLUTIONS: Exhibit = Exhibit {
         }
 
         Ok(space)
-    },
-};
+    }
+}
 
-const ROTATIONS: Exhibit = Exhibit {
+exhibit! {
+    const ROTATIONS,
     name: "Rotations",
-    factory: |_this, universe| {
+    (_this, universe) {
         let demo_blocks = BlockProvider::<DemoBlocks>::using(universe)?;
         let mut space = Space::empty(Grid::new([-2, 0, -2], [5, 5, 5]));
 
@@ -386,12 +494,13 @@ const ROTATIONS: Exhibit = Exhibit {
         }
 
         Ok(space)
-    },
-};
+    }
+}
 
-const COLORS: Exhibit = Exhibit {
+exhibit! {
+    const COLORS,
     name: "Colors",
-    factory: |_this, universe| {
+    (_this, universe) {
         let gradient_resolution = 5;
         let mut space = Space::empty(Grid::new(
             [0, 0, 0],
@@ -410,7 +519,7 @@ const COLORS: Exhibit = Exhibit {
                     .to_vec()
                     .map(|s| NotNan::new(s as f32 / (gradient_resolution - 1) as f32).unwrap()),
             );
-            let color_srgb = color.with_alpha_one().to_srgb_32bit();
+            let color_srgb = color.with_alpha_one().to_srgb8();
             let description = format!(
                 "Linear\n  {:0.2}\n  {:0.2}\n  {:0.2}\nsRGB\n  #{:02x}{:02x}{:02x}",
                 color.red(),
@@ -455,18 +564,27 @@ const COLORS: Exhibit = Exhibit {
         })?;
 
         Ok(space)
-    },
-};
+    }
+}
 
-const COLOR_LIGHTS: Exhibit = Exhibit {
+exhibit! {
+    const COLOR_LIGHTS,
     name: "Colored Lights",
-    factory: |_this, universe| {
-        let room_width = 7;
-        let room_length = 12;
-        let room_height = 5;
+    (_this, universe) {
+        let room_width = 11;
+        let room_length = 16;
+        let room_height = 7;
+        let separator_width = 4;  // less than room_width/2
+        let brightness = 1.0;
+
         let interior = Grid::new([0, 0, 0], [room_width, room_height, room_length]);
         let mut space = Space::empty(interior.expand(FaceMap::from_fn(|_| 1)));
 
+
+        fn normalize(color: Rgb) -> Rgb {
+            color * color.luminance().recip()
+        }
+
         let light_colors = [
             rgb_const!(1.0, 0.0, 0.0),
             rgb_const!(1.0, 1.0, 0.0),
@@ -520,7 +638,8 @@ const COLOR_LIGHTS: Exhibit = Exhibit {
 
         // Wall corner
         let corner = Block::builder()
-            .display_name("Color room wall")
+            .display_name("Color room wall corner")
+            .rotation_rule(RotationPlacementRule::Attach { by: Face::NZ }) // TODO: more specific
             .voxels_fn(universe, wall_resolution, |p| {
                 if p.x.pow(2) + p.z.pow(2) < GridCoordinate::from(wall_resolution).pow(2) {
                     &wall_color_block
@@ -553,7 +672,6 @@ const COLOR_LIGHTS: Exhibit = Exhibit {
         )?;
 
         // Vertical separators
-        let separator_width = 2;
         space.fill_uniform(
             Grid::new([0, room_height / 2, 0], [separator_width, 1, room_length]),
             &wall_block,
@@ -572,7 +690,7 @@ const COLOR_LIGHTS: Exhibit = Exhibit {
             &AIR,
         )?;
 
-        // Place lights
+        // Place lights and horizontal separators
         for (i, color) in light_colors.iter().copied().enumerate() {
             let z = (i as GridCoordinate) * (room_length - 1)
                 / (light_colors.len() as GridCoordinate - 1);
@@ -582,7 +700,7 @@ const COLOR_LIGHTS: Exhibit = Exhibit {
                 Block::builder()
                     .display_name("Colored light with colored surface")
                     .color(color.with_alpha_one())
-                    .light_emission(color * 1.2)
+                    .light_emission(normalize(color) * brightness)
                     .build(),
             )?;
             space.set(
@@ -590,32 +708,43 @@ const COLOR_LIGHTS: Exhibit = Exhibit {
                 Block::builder()
                     .display_name("Colored light with white surface")
                     .color(Rgba::WHITE)
-                    .light_emission(color * 10.0)
+                    .light_emission(normalize(color) * brightness)
                     .build(),
             )?;
+
+            // Separator between different light areas
+            if true || i > 0  {
+                if i % 2 == 0 {
+                    space.fill_uniform(Grid::new([room_width - separator_width, 0, z], [separator_width, room_height, 1]), &wall_block)?;
+                } else {
+                    space.fill_uniform(Grid::new([0, 0, z], [separator_width, room_height, 1]), &wall_block)?;
+                }
+            }
         }
 
         // TODO: Add an RGBCMY section, and also a color-temperature section (or maybe different buildings)
         // sRGB white is D65, or approximately 6500 K.
 
         Ok(space)
-    },
-};
+    }
+}
 
-const CHUNK_CHART: Exhibit = Exhibit {
+exhibit! {
+    const CHUNK_CHART,
     name: "Visible chunk chart",
-    factory: |_this, _universe| {
+    (_this, _universe) {
         use all_is_cubes::chunking::ChunkChart;
 
         // TODO: Show more than one size.
         let chart = ChunkChart::<16>::new(16. * 4.99);
         Ok(chart.visualization())
-    },
-};
+    }
+}
 
-const MAKE_SOME_BLOCKS: Exhibit = Exhibit {
+exhibit! {
+    const MAKE_SOME_BLOCKS,
     name: "make_some_blocks",
-    factory: |_this, mut universe| {
+    (_this, universe) {
         const ROWS: GridCoordinate = 5;
         fn make_both_blocks<const N: usize>(universe: &mut Universe) -> (Vec<Block>, Vec<Block>) {
             (
@@ -624,26 +753,27 @@ const MAKE_SOME_BLOCKS: Exhibit = Exhibit {
             )
         }
         let rows: [(Vec<Block>, Vec<Block>); ROWS as usize] = [
-            make_both_blocks::<5>(&mut universe),
-            make_both_blocks::<4>(&mut universe),
-            make_both_blocks::<3>(&mut universe),
-            make_both_blocks::<2>(&mut universe),
-            make_both_blocks::<1>(&mut universe),
+            make_both_blocks::<5>(universe),
+            make_both_blocks::<4>(universe),
+            make_both_blocks::<3>(universe),
+            make_both_blocks::<2>(universe),
+            make_both_blocks::<1>(universe),
         ];
         let mut space = Space::empty_positive(3, ROWS, ROWS);
-        for (y, (blocks_a, blocks_v)) in std::array::IntoIter::new(rows).enumerate() {
+        for (y, (blocks_a, blocks_v)) in rows.into_iter().enumerate() {
             for (h, (block_a, block_v)) in blocks_a.into_iter().zip(blocks_v).enumerate() {
                 space.set([0, y as GridCoordinate, h as GridCoordinate], block_a)?;
                 space.set([2, y as GridCoordinate, h as GridCoordinate], block_v)?;
             }
         }
         Ok(space)
-    },
-};
+    }
+}
 
-const SWIMMING_POOL: Exhibit = Exhibit {
+exhibit! {
+    const SWIMMING_POOL,
     name: "Swimming Pool",
-    factory: |_this, _universe| {
+    (_this, _universe) {
         let width = 6;
         let depth = 6;
         let water_area = Grid::new([0, -depth, 0], [width, depth, width]);
@@ -657,5 +787,5 @@ const SWIMMING_POOL: Exhibit = Exhibit {
                 .build(),
         )?;
         Ok(space)
-    },
-};
+    }
+}
